function xUL_f2d_xM_t0tau(dxdt_fnc,xgrd,f0,t_op,tau_op,ido_bwd)
% xUL_f2d_xM_t0tau: compute x and M
%   input
%      dxdt_fnc:    flow model
%      xgrd:        tracer ic
%      f0:          flow ic
%      t_op:        t0
%      tau_op:      tau
%      ido_bwd:     do or do not backward computation
%   output into dfnm_op.x,M,f
%      
% Written by Kayo Ide (University of Maryland)
% 2015 August

%% global
% system input (passed on)
global Nd
global ido idont
% 
global ixMf_strt ixMf_end
global ido_xMf
% 
global dfnm_op

%% set up 
cpu0=cputime;
% xMf0: x & M initialization
ido_xMf=ido;
%- x on spatial grid
[Xgrd1,Xgrd2]=meshgrid(linspace(xgrd(1).mn,xgrd(1).mx,xgrd(1).n),...
    linspace(xgrd(2).mn,xgrd(2).mx,xgrd(2).n));
%- M on spatial grid
Mgrd=zeros(size(Xgrd1));
Np=numel(Mgrd);
%- xMf0
xMf0=[Xgrd1(:); Xgrd2(:); Mgrd(:); f0(:)];
% tau: temporal resolution
Ktau=numel(tau_op);
% output: xfwd/bwd & Mfwd/bwd 
%         xvfwd/bwd
xfwd=zeros(Np,Ktau,Nd);  Mfwd=zeros(Np,Ktau); 
ffwd=zeros(numel(f0(:)),Ktau);
if(ido_bwd-ido==0); xbwd=xfwd; Mbwd=Mfwd; fbwd=ffwd;
else; xbwd=0; Mbwd=0; fbwd=0;
end

%% Computation of M
% forward
fprintf('\n        Mfwd at t0=%2.2f up to tau(1:%d)=[%2.2f,%2.2f]',...
    t_op(1),Ktau,tau_op(1),tau_op(end));
k=1; xMf0k=[xMf0]';
for k=1:Ktau;  
    if(k==1);
        xMfk=dxdt_fnc(t_op(k),xMf0);
        xfwd(:,k,1)=Xgrd1(:);
        xfwd(:,k,2)=Xgrd2(:);
        for nd=1:Nd
            Mfwd(:,k)=Mfwd(:,k)+xMfk(ixMf_strt.x(nd):ixMf_end.x(nd)).^2;
        end
        Mfwd(:,k)=sqrt(Mfwd(:,k));
        ffwd(:,k)=f0(:)';
    else;
        km1=k-1;
        [tk,xMfk]=ode45(dxdt_fnc,t_op(k)+[tau_op(km1) tau_op(k)],xMf0k);
        xMf0k=xMfk(end,:);
        for nd=1:Nd
            xfwd(:,k,nd)=xMf0k([ixMf_strt.x(nd):ixMf_end.x(nd)])';
        end
        Mfwd(:,k)=xMf0k(ixMf_strt.M:ixMf_end.M)';
        ffwd(:,k)=xMf0k(ixMf_strt.f(1):end)';
    end    
    fprintf('\n               tau=%2.2f',tau_op(k));
end

% backword
if(ido_bwd-ido==0); 
    fprintf('\n        Mbwd at t0=%2.2f up to tau(1:%d)=[-%2.2f,-%2.2f]',...
        t_op(1),Ktau,tau_op(1),tau_op(end));
    xMf0k=[xMf0]';
    for k=1:Ktau; 
        if(k==1); 
            for nd=1:Nd
                xbwd(:,k,nd)=xfwd(:,k,nd);
            end            
            Mbwd(:,k)=-Mfwd(:,k);
            fbwd(:,k)= ffwd(:,k);
        else;
            km1=k-1;
            [tk,xMfk]=ode45(dxdt_fnc,t_op(k)-[tau_op(km1) tau_op(k)],xMf0k);
            xMf0k=xMfk(end,:);
            for nd=1:Nd
                xbwd(:,k,nd)=xMf0k([ixMf_strt.x(nd):ixMf_end.x(nd)])';
            end
            Mbwd(:,k)=xMf0k([ixMf_strt.M:ixMf_end.M])';
            fbwd(:,k)=xMf0k([ixMf_strt.f(1):end])';
        end
        fprintf('\n               tau=%2.2f',-tau_op(k));
    end
end

%% output
xgrd_op=xgrd;
save(dfnm_op.x,'-mat','t_op','tau_op','xgrd_op','xfwd','xbwd')
save(dfnm_op.M,'-mat','t_op','tau_op','xgrd_op','Mfwd','Mbwd')
save(dfnm_op.f,'-mat','t_op','tau_op','xgrd_op','ffwd','fbwd')

fprintf('\n        cpu for xM=%10.2f',cputime-cpu0);

return